package be.jeffcheasey88.peeratcode.parser.java;

import java.util.ArrayList;
import java.util.List;
import java.util.function.Function;

import be.jeffcheasey88.peeratcode.parser.Bag;
import be.jeffcheasey88.peeratcode.parser.Token;
import be.jeffcheasey88.peeratcode.parser.java.Annotation.AnnotableBuffer;

public class JavaFile extends JavaElement implements ClassContainer, AnnotableBuffer{
	
	private List<Annotation> annotationBuffer;
	
	private Token pack;
	private List<Import> imports;
	private Class mainClazz;
	private List<Class> subClazz;
	
	public JavaFile(){
		this.imports = new ArrayList<>();
		this.subClazz = new ArrayList<>();
	}
	
	JavaFile setPackage(Bag bag){
		this.pack = bag.<Token>get();
		return this;
	}
	
	JavaFile addImport(Import value){
		this.imports.add(value);
		return this;
	}
	
	@Override
	public void addClass(Class clazz){
		if(mainClazz == null) this.mainClazz = clazz;
		else subClazz.add(clazz);
	}
	
	public Token getPackage(){
		return this.pack;
	}
	
	public List<Import> getImports(){
		return this.imports;
	}
	
	public Class getMainClass(){
		return this.mainClazz;
	}
	
	public List<Class> getSubClasses(){
		return this.subClazz;
	}

	@Override
	public <E extends JavaElement> E find(Function<JavaElement, Boolean> finder){
		if(finder.apply(mainClazz)) return (E) mainClazz;
		for(Class clazz : subClazz){
			if(finder.apply(clazz)) return (E) clazz;
		}
		return null;
	}

	@Override
	public void addAnnotationBuffer(Annotation annotation){
		if(annotationBuffer == null) annotationBuffer = new ArrayList<>();
		annotationBuffer.add(annotation);
	}
	
	public List<Annotation> getAnnotationBuffer(){
		List<Annotation> list = this.annotationBuffer;
		this.annotationBuffer = null;
		return list;
	}
}
